/*
 * Copyright 2011 Apaq.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package dk.apaq.crud;

import dk.apaq.filter.Filter;
import dk.apaq.filter.limit.Limit;
import dk.apaq.filter.sort.Sorter;
import java.util.List;

/**
 * An Interface for a Crud. The interface is split into several interfaces allowing
 * for Crud's to only implement some methods fx. for readonly cruds.
 *
 * If more functionality is required, fx. update, create, delete or filtering, then
 * the cruds need to implement the subinterfaces.
 * @param <IDT> The type used as Id for the entity, fx. String.
 * @param <BT> The entity type.
 */
public interface Crud<IDT,T> {

    /**
     * Reads a single enntity by its id.
     * @param id The id of the entity to read.
     * @return Returns the entity or null if entity available with the specified id.
     */
    T read(IDT id);

    /**
     * Lists all id's.
     * @return The list of all id's
     */
    List<IDT> listIds();

    /**
     * Lists all id's.
     * @param limit Limits the returned result
     * @return The list of all id's
     */
    List<IDT> listIds(Limit limit);


    /**
     * Interface for editable cruds.
     * @param <IDT> The type used as Id for the entity, fx. String.
     * @param <BT> The entity type.
     */
    public interface Editable<IDT,BT> extends Crud<IDT,BT> {
        IDT create();
        void update(BT entity);
        void delete(IDT id);
    }

    /**
     * Interface for filterable cruds.
     * @param <IDT> The type used as Id for the entity, fx. String.
     * @param <BT> The entity type.
     */
    public interface Filterable<IDT,BT> extends Crud<IDT,BT> {
        List<IDT> listIds(Filter filter, Sorter sorter);
        List<IDT> listIds(Filter filter, Sorter sorter, Limit limit);
    }

    /**
     * Interface for cruds that implements all crud methods.
     * @param <IDT> The type used as Id for the entity, fx. String.
     * @param <BT> The entity type.
     */
    public interface Complete<IDT,BT> extends Filterable<IDT,BT>, Editable<IDT, BT> { }
}
