/*
 * Copyright 2011 Apaq.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package dk.apaq.crud.core;

import dk.apaq.crud.Crud;
import dk.apaq.crud.CrudEvent;
import dk.apaq.crud.CrudListener;
import dk.apaq.crud.CrudNotifier;
import dk.apaq.crud.HasId;
import java.util.ArrayList;
import java.util.List;

/**
 * An abstract implementation of a <code>Crud.Editable</code> instance taking care
 * of CrudNotifer logic. An implementing class needs only to call the fire* methods(fx. fireOnRead)
 * and this class will make sure to fire evetns on the listeners.
 */
public abstract class BaseCrud<IDT, BT> implements
        Crud<IDT, BT>, CrudNotifier {

    protected final List<CrudListener<IDT, BT>> listeners = new ArrayList<CrudListener<IDT, BT>>();

    @Override
    public void addListener(CrudListener listener) {
        listeners.add(listener);
    }

    @Override
    public void removeListener(CrudListener listener) {
        listeners.remove(listener);
    }

    /**
     * Tells listeners that an entity has been read from crud.
     * @param entity The entity that was read.
     */
    protected void fireOnRead(IDT id, BT entity) {
        CrudEvent.WithEntity e = createEventWithEntity(id, entity);
        for (CrudListener listener : listeners) {
            listener.onEntityRead(e);
        }
    }

    /**
     * Tells listeners that an entity is about tobe read.
     * @param id The id of the entity that is to be read.
     */
    protected void fireOnBeforeRead(IDT id) {
        CrudEvent.WithId e = createEventWithId(id);
        for (CrudListener listener : listeners) {
            listener.onBeforeEntityRead(e);
        }
    }

    

    protected CrudEvent createEvent() {
        return new CrudEvent(this);
    }

    protected CrudEvent.WithId<IDT, BT> createEventWithId(IDT id) {
        return new CrudEvent.WithId<IDT, BT>(this, id);
    }

    protected CrudEvent.WithEntity<IDT, BT> createEventWithEntity(IDT id, BT entity) {
        return new CrudEvent.WithEntity<IDT, BT>(this, id, entity);
    }

}
